using System;
using System.Collections.Generic;
using System.Linq;

namespace Jade.DataMining.Classifier
{
    public class NaiveBayes : ModelTraining
    {
        public NaiveBayes(bool loadFromFile = false)
        {
            this.IsTrain = loadFromFile;
            if (loadFromFile)
            {
                this.Load();
            }
        }

        /// <summary>
        ///  vTable[termId][Category]
        /// </summary>
        public List<List<double>> vTable { get; set; }

        /// <summary>
        /// cPrior[classid] 
        /// </summary>
        public List<double> Prior { get; set; }

        public void Train(Matrix matrix)
        {
            if (matrix.RowsCount != matrix.Categories.Count)
            {
                throw new Exception("ERROR!,matrix.RowsCount shoud be equal to matrix.Categories.Count");
            }

            //  #calculate prior of each class
            //  #1. init cPrior:

            var distinctCategories = matrix.Categories.Distinct().ToList();
            distinctCategories.Sort();
            var cPrior = new double[distinctCategories.Count + 1];

            // 2. fill cPrior
            matrix.Categories.ForEach(classid => cPrior[classid] += 1);

            //#calculate likehood of each term
            // #1. init vTable:  vTable[termId][Category]
            vTable = new List<List<double>>();
            for (var i = 0; i < matrix.ColsCount; i++)
            {
                var data = cPrior.Select(t => 0d).ToList();
                vTable.Add(data);
            }

            // #2. fill vTable
            for (var i = 0; i < matrix.RowsCount; i++)
            {
                for (var j = matrix.Rows[i]; j < matrix.Rows[i + 1]; j++)
                {
                    vTable[matrix.Cols[j]][matrix.Categories[i]] += 1;
                }
            }

            //#normalize vTable
            for (var i = 0; i < matrix.ColsCount; i++)
            {
                for (var j = 0; j < cPrior.Length; j++)
                {
                    // P(x|c) =  term  /   
                    if (cPrior[j] > 1e-10)
                        vTable[i][j] /= (cPrior[j]);
                }
            }

            //#normalize cPrior P(C) = C/TC
            for (var i = 0; i < cPrior.Length; i++)
            {
                cPrior[i] /= matrix.Categories.Count;
            }

            this.Prior = new List<double>(cPrior);

            this.IsTrain = true;

            this.Save();

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="matrix"></param>
        /// <returns></returns>
        public string TestSample(Matrix matrix)
        {
            var targetP = new List<double>();
            var maxP = -1000000000d;
            var best = -1;
            // P(C)*P(X|C)
            for (var target = 0; target < this.Prior.Count; target++)
            {
                var curP = 100D; // Ŵ100
                curP *= this.Prior[target];

                for (var c = 0; c < matrix.Cols.Count; c++)
                {
                    if (this.vTable[matrix.Cols[c]][target] == 0)
                    {
                        curP *= 1e-7;
                    }
                    else
                    {
                        curP *= vTable[matrix.Cols[c]][target];
                    }
                }
                targetP.Add(curP);
                if (curP > maxP)
                {
                    best = target;
                    maxP = curP;
                }
            }

            return GlobalInfo.Instance.IdToClass[best];

        }


        /// <summary>
        /// Ҷ˹ģ
        /// </summary>
        [Serializable]
        public class NaiveBayesModel
        {
            public List<List<double>> vTable { get; set; }
            public List<double> Prior { get; set; }
        }

        public override void Save()
        {
            try
            {
                var model = new NaiveBayesModel { vTable = this.vTable, Prior = this.Prior };
                SerializeHelper helper = new SerializeHelper();
                helper.ToBinaryFile(model, ClassiferSetting.NaiveBayesSetting.ModelPath);
            }
            catch
            {
                Console.WriteLine("ؿģʧ");
            }
        }

        public override void Load()
        {
            try
            {
                Console.WriteLine("رҶ˹ģ͡");
                SerializeHelper helper = new SerializeHelper();
                var model = (NaiveBayesModel)helper.FromBinaryFile<NaiveBayesModel>(ClassiferSetting.NaiveBayesSetting.ModelPath);
                this.vTable = model.vTable;
                this.Prior = model.Prior;
            }
            catch
            {
                Console.WriteLine("رҶ˹ģʧ");
            }
        }
    }
}