﻿using System.IO;
using System.Xml;

namespace Jade.DataMining.Classifier
{
    /// <summary>
    /// 分类器设置
    /// </summary>
    public class ClassiferSetting
    {
        static ClassiferSetting()
        {
            if(!Directory.Exists("model"))
            {
                Directory.CreateDirectory("model");
            }

            var configFile = "Config.xml";
            if (File.Exists(configFile))
            {
                XmlDocument config = new XmlDocument();
                config.Load(configFile);
                var global = config.GetElementsByTagName("__global__")[0];
                GlobalSetting = new GlobalSetting()
                                    {
                                        ClassToDocCount = global.SelectSingleNode("class_to_doc_count").InnerText,
                                        IdToDocCount = global.SelectSingleNode("class_to_doc_count").InnerText,
                                        IdToIdf = global.SelectSingleNode("id_to_idf").InnerText,
                                        IdToTerm = global.SelectSingleNode("id_to_term").InnerText,
                                        NewidToId = global.SelectSingleNode("newid_to_id").InnerText,
                                        TermToId = global.SelectSingleNode("term_to_id").InnerText,
                                        ClassToId = global.SelectSingleNode("class_to_id").InnerText,
                                        IdToClass = global.SelectSingleNode("id_to_class").InnerText
                                    };
                var naive_bayes = config.GetElementsByTagName("naive_bayes")[0];
                NaiveBayesSetting = new NaiveBayesSetting()
                                        {
                                            LogPath = naive_bayes.SelectSingleNode("log_path").InnerText,
                                            ModelPath = naive_bayes.SelectSingleNode("model_path").InnerText
                                        };
                var filter = config.GetElementsByTagName("__filter__")[0];

                FilterSetting = new FilterSetting()
                                    {
                                        Rate = double.Parse(filter.SelectSingleNode("rate").InnerText),
                                        Method = filter.SelectSingleNode("method").InnerText,
                                        LogPath = filter.SelectSingleNode("log_path").InnerText,
                                        ModelPath = filter.SelectSingleNode("model_path").InnerText

                                    };
            }

            try
            {
                LoadExistModel = bool.Parse(System.Configuration.ConfigurationManager.AppSettings["LoadExistModel"]);
            }
            catch
            {
                LoadExistModel = false;
            }
        }

        public static GlobalSetting GlobalSetting { get; private set; }

        public static bool LoadExistModel { get; set; }

        public static FilterSetting FilterSetting { get; private set; }

        public static NaiveBayesSetting NaiveBayesSetting { get; private set; }
    }
}
