﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Security.Cryptography;
using System.Text;
using System.Text.RegularExpressions;
using System.Web;

namespace Jade.DataMining
{
    public static class EnumerableExtension
    {
        /// <summary>
        /// 遍历集合(foreach 不修改对象)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="enumerable"></param>
        /// <param name="action"></param>
        [DebuggerStepThrough]
        public static void ForEach<T>(this IEnumerable<T> enumerable, Action<T> action)
        {
            foreach (T item in enumerable)
            {
                action(item);
            }
        }

        /// <summary>
        /// 遍历集合(for 可以修改对象)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="enumerable"></param>
        /// <param name="action"></param>
        [DebuggerStepThrough]
        public static void ForLoop<T>(this IList<T> enumerable, Action<T> action)
        {
            for (var i = 0; i < enumerable.Count; i++)
            {
                action(enumerable[i]);
            }
        }
    }

    public static class StringExtension
    {
        #region stopWords
        static string stopWords = @"a
a's

able

about

above

according

accordingly

across

actually

after

afterwards

again

against

ain't

all

allow

allows

almost

alone

along

already

also

although

always

am

among

amongst

an

and

another

any

anybody

anyhow

anyone

anything

anyway

anyways

anywhere

apart

appear

appreciate

appropriate

are

aren't

around

as

aside

ask

asking

associated

at

available

away

awfully

b

be

became

because

become

becomes

becoming

been

before

beforehand

behind

being

believe

below

beside

besides

best

better

between

beyond

both

brief

but

by

c

c'mon

c's

came

can

can't

cannot

cant

cause

causes

certain

certainly

changes

clearly

co

com

come

comes

concerning

consequently

consider

considering

contain

containing

contains

corresponding

could

couldn't

course

currently

d

definitely

described

despite

did

didn't

different

do

does

doesn't

doing

don't

done

down

downwards

during

e

each

edu

eg

eight

either

else

elsewhere

enough

entirely

especially

et

etc

even

ever

every

everybody

everyone

everything

everywhere

ex

exactly

example

except

f

far

few

fifth

first

five

followed

following

follows

for

former

formerly

forth

four

from

further

furthermore

g

get

gets

getting

given

gives

go

goes

going

gone

got

gotten

greetings

h

had

hadn't

happens

hardly

has

hasn't

have

haven't

having

he

he's

hello

help

hence

her

here

here's

hereafter

hereby

herein

hereupon

hers

herself

hi

him

himself

his

hither

hopefully

how

howbeit

however

i

i'd

i'll

i'm

i've

ie

if

ignored

immediate

in

inasmuch

inc

indeed

indicate

indicated

indicates

inner

insofar

instead

into

inward

is

isn't

it

it'd

it'll

it's

its

itself

j

just

k

keep

keeps

kept

know

knows

known

l

last

lately

later

latter

latterly

least

less

lest

let

let's

like

liked

likely

little

look

looking

looks

ltd

m

mainly

many

may

maybe

me

mean

meanwhile

merely

might

more

moreover

most

mostly

much

must

my

myself

n

name

namely

nd

near

nearly

necessary

need

needs

neither

never

nevertheless

new

next

nine

no

nobody

non

none

noone

nor

normally

not

nothing

novel

now

nowhere

o

obviously

of

off

often

oh

ok

okay

old

on

once

one

ones

only

onto

or

other

others

otherwise

ought

our

ours

ourselves

out

outside

over

overall

own

p

particular

particularly

per

perhaps

placed

please

plus

possible

presumably

probably

provides

q

que

quite

qv

r

rather

rd

re

really

reasonably

regarding

regardless

regards

relatively

respectively

right

s

said

same

saw

say

saying

says

second

secondly

see

seeing

seem

seemed

seeming

seems

seen

self

selves

sensible

sent

serious

seriously

seven

several

shall

she

should

shouldn't

since

six

so

some

somebody

somehow

someone

something

sometime

sometimes

somewhat

somewhere

soon

sorry

specified

specify

specifying

still

sub

such

sup

sure

t

t's

take

taken

tell

tends

th

than

thank

thanks

thanx

that

that's

thats

the

their

theirs

them

themselves

then

thence

there

there's

thereafter

thereby

therefore

therein

theres

thereupon

these

they

they'd

they'll

they're

they've

think

third

this

thorough

thoroughly

those

though

three

through

throughout

thru

thus

to

together

too

took

toward

towards

tried

tries

truly

try

trying

twice

two

u

un

under

unfortunately

unless

unlikely

until

unto

up

upon

us

use

used

useful

uses

using

usually

uucp

v

value

various

very

via

viz

vs

w

want

wants

was

wasn't

way

we

we'd

we'll

we're

we've

welcome

well

went

were

weren't

what

what's

whatever

when

whence

whenever

where

where's

whereafter

whereas

whereby

wherein

whereupon

wherever

whether

which

while

whither

who

who's

whoever

whole

whom

whose

why

will

willing

wish

with

within

without

won't

wonder

would

would

wouldn't

x

y

yes

yet

you

you'd

you'll

you're

you've

your

yours

yourself

yourselves

z

zero
about

above

across

after

again

against

all

almost

alone

along

already

also

although

always

among

an

and

another

any

anybody

anyone

anything

anywhere

are

area

areas

around

as

ask

asked

asking

asks

at

away

b

back

backed

backing

backs

be

became

because

become

becomes

been

before

began

behind

being

beings

best

better

between

big

both

but

by

c

came

can

cannot

case

cases

certain

certainly

clear

clearly

come

could

d

did

differ

different

differently

do

does

done

down

down

downed

downing

downs

during

e

each

early

either

end

ended

ending

ends

enough

even

evenly

ever

every

everybody

everyone

everything

everywhere

f

face

faces

fact

facts

far

felt

few

find

finds

first

for

four

from

full

fully

further

furthered

furthering

furthers

g

gave

general

generally

get

gets

give

given

gives

go

going

good

goods

got

great

greater

greatest

group

grouped

grouping

groups

h

had

has

have

having

he

her

here

herself

high

high

high

higher

highest

him

himself

his

how

however

i

if

important

in

interest

interested

interesting

interests

into

is

it

its

itself

j

just

k

keep

keeps

kind

knew

know

known

knows

l

large

largely

last

later

latest

least

less

let

lets

like

likely

long

longer

longest

m

made

make

making

man

many

may

me

member

members

men

might

more

most

mostly

mr

mrs

much

must

my

myself

n

necessary

need

needed

needing

needs

never

new

new

newer

newest

next

no

nobody

non

noone

not

nothing

now

nowhere

number

numbers

o

of

off

often

old

older

oldest

on

once

one

only

open

opened

opening

opens

or

order

ordered

ordering

orders

other

others

our

out

over

p

part

parted

parting

parts

per

perhaps

place

places

point

pointed

pointing

points

possible

present

presented

presenting

presents

problem

problems

put

puts

q

quite

r

rather

really

right

right

room

rooms

s

said

same

saw

say

says

second

seconds

see

seem

seemed

seeming

seems

sees

several

shall

she

should

show

showed

showing

shows

side

sides

since

small

smaller

smallest

so

some

somebody

someone

something

somewhere

state

states

still

still

such

sure

t

take

taken

than

that

the

their

them

then

there

therefore

these

they

thing

things

think

thinks

this

those

though

thought

thoughts

three

through

thus

to

today

together

too

took

toward

turn

turned

turning

turns

two

u

under

until

up

upon

us

use

used

uses

v

very

w

want

wanted

wanting

wants

was

way

ways

we

well

wells

went

were

what

when

where

whether

which

while

who

whole

whose

why

will

with

within

without

work

worked

working

works

would

x

y

year

years

yet

you

young

younger

youngest

your

yours

z

";
        #endregion


        public static string GetSummary(this string source, int length)
        {
            if (source.Length < length)
            {
                return source;
            }
            return source.Substring(0, length) + "...";
        }

        /// <summary>
        /// 移除英文停用词
        /// </summary>
        /// <param name="source"></param>
        /// <returns></returns>
        public static string ExcludeStopWords(this string source)
        {
            var words = stopWords.Split(new string[] { "\r\n" }, StringSplitOptions.RemoveEmptyEntries);

            source = source.Replace(new List<string> { "\"", "*", "%", "$", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "?", "(", ")", "-", "&amp;", "=", "&quot;" }, string.Empty).ToLower();

            var sourceWords = new List<string>(source.Split(new string[] { " ", ",", ".", "'", "\"" }, StringSplitOptions.RemoveEmptyEntries));

            foreach (var word in words)
            {
                if (sourceWords.Contains(word))
                {
                    sourceWords.Remove(word);
                }
            }

            return string.Join(" ", sourceWords.ToArray());
        }

        /// <summary>
        /// 创建目录
        /// </summary>
        /// <param name="dir"></param>
        [DebuggerStepThrough]
        public static void CreateDirIfNotExist(this string dir)
        {
            if (!Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }
        }


        [DebuggerStepThrough]
        public static bool IsExistDir(this string dir)
        {
            return Directory.Exists(dir);
        }

        public static string GetSafeFileName(this string fileName)
        {
            var chars = System.IO.Path.GetInvalidFileNameChars();

            foreach (var c in chars)
            {
                fileName = fileName.Replace(c, ' ');
            }

            return fileName.Replace(" ", "");
        }

        [DebuggerStepThrough]
        public static string Format(this string format, params object[] args)
        {
            return string.Format(format, args);
        }

        private static readonly Regex WebUrlExpression = new Regex(
            @"(http|https)://([\w-]+\.)+[\w-]+(/[\w- ./?%&=]*)?", RegexOptions.Singleline | RegexOptions.Compiled);

        private static readonly Regex EmailExpression =
            new Regex(@"^([0-9a-zA-Z]+[-._+&])*[0-9a-zA-Z]+@([-0-9a-zA-Z]+[.])+[a-zA-Z]{2,6}$",
                      RegexOptions.Singleline | RegexOptions.Compiled);

        private static readonly Regex StripHTMLExpression = new Regex("<\\S[^><]*>",
                                                                      RegexOptions.IgnoreCase | RegexOptions.Singleline |
                                                                      RegexOptions.Multiline |
                                                                      RegexOptions.CultureInvariant |
                                                                      RegexOptions.Compiled);

        private static readonly char[] IllegalUrlCharacters = new[]
                                                                  {
                                                                      ';', '/', '\\', '?', ':', '@', '&', '=', '+', '$',
                                                                      ',', '<', '>', '#', '%', '.', '!', '*', '\'', '"',
                                                                      '(', ')', '[', ']', '{', '}', '|', '^', '`', '~', '–'
                                                                      , '‘', '’', '“', '”', '»', '«'
                                                                  };

        [DebuggerStepThrough]
        public static bool IsWebUrl(this string target)
        {
            return !string.IsNullOrEmpty(target) && WebUrlExpression.IsMatch(target);
        }

        [DebuggerStepThrough]
        public static bool IsEmail(this string target)
        {
            return !string.IsNullOrEmpty(target) && EmailExpression.IsMatch(target);
        }

        [DebuggerStepThrough]
        public static string NullSafe(this string target)
        {
            return (target ?? string.Empty).Trim();
        }

        [DebuggerStepThrough]
        public static string FormatWith(this string target, params object[] args)
        {
            //  Check.Argument.IsNotEmpty(target, "target");

            return string.Format(target, args);
        }

        [DebuggerStepThrough]
        public static string Hash(this string target)
        {
            //  Check.Argument.IsNotEmpty(target, "target");

            using (MD5 md5 = MD5.Create())
            {
                byte[] data = Encoding.Unicode.GetBytes(target);
                byte[] hash = md5.ComputeHash(data);

                return Convert.ToBase64String(hash);
            }
        }

        [DebuggerStepThrough]
        public static string WrapAt(this string target, int index)
        {
            const int DotCount = 3;

            //  Check.Argument.IsNotEmpty(target, "target");
            //  Check.Argument.IsNotNegativeOrZero(index, "index");

            return (target.Length <= index)
                       ? target
                       : string.Concat(target.Substring(0, index - DotCount), new string('.', DotCount));
        }

        [DebuggerStepThrough]
        public static string StripHtml(this string target)
        {
            return StripHTMLExpression.Replace(target, string.Empty);
        }

        [DebuggerStepThrough]
        public static Guid ToGuid(this string target)
        {
            return new Guid(target);
        }

        [DebuggerStepThrough]
        public static T ToEnum<T>(this string target, T defaultValue) where T : IComparable, IFormattable
        {
            T convertedValue = defaultValue;

            if (!string.IsNullOrEmpty(target))
            {
                try
                {
                    convertedValue = (T)Enum.Parse(typeof(T), target.Trim(), true);
                }
                catch (ArgumentException)
                {
                }
            }

            return convertedValue;
        }

        [DebuggerStepThrough]
        public static string ToLegalUrl(this string target)
        {
            if (string.IsNullOrEmpty(target))
            {
                return target;
            }

            target = target.Trim();

            if (target.IndexOfAny(IllegalUrlCharacters) > -1)
            {
                foreach (char character in IllegalUrlCharacters)
                {
                    target = target.Replace(character.ToString(), string.Empty);
                }
            }

            target = target.Replace(" ", "-");

            while (target.Contains("--"))
            {
                target = target.Replace("--", "-");
            }

            return target;
        }

        [DebuggerStepThrough]
        public static string UrlEncode(this string target)
        {
            return HttpUtility.UrlEncode(target);
        }

        [DebuggerStepThrough]
        public static string UrlDecode(this string target)
        {
            return HttpUtility.UrlDecode(target);
        }

        [DebuggerStepThrough]
        public static string AttributeEncode(this string target)
        {
            return HttpUtility.HtmlAttributeEncode(target);
        }

        [DebuggerStepThrough]
        public static string HtmlEncode(this string target)
        {
            return HttpUtility.HtmlEncode(target);
        }

        [DebuggerStepThrough]
        public static string HtmlDecode(this string target)
        {
            return HttpUtility.HtmlDecode(target);
        }

        public static string Replace(this string target, ICollection<string> oldValues, string newValue)
        {
            oldValues.ForEach(oldValue => target = target.Replace(oldValue, newValue));
            return target;
        }

        /// <summary>
        ///   过滤html危险脚本
        /// </summary>
        /// <param name = "target"></param>
        /// <returns></returns>
        public static string HtmlSanitizer(this string html)
        {
            Regex regex1 = new Regex(@"<(script|link|style|iframe|button|input|frameset)(.|\n)*\/\1>\BelongsTo*", RegexOptions.IgnoreCase);
            Regex regex2 = new Regex(@" href *= *[\BelongsTo\S]*script *:", RegexOptions.IgnoreCase);
            Regex regex3 = new Regex(@" no[\BelongsTo\S]*=", RegexOptions.IgnoreCase);
            //Regex regex6 = new Regex(@"\<img[^\>]+\>",RegexOptions.IgnoreCase);
            html = regex1.Replace(html, ""); //过滤危险标记 
            html = regex2.Replace(html, ""); //过滤href=javascript: (<A>) 属性 
            html = regex3.Replace(html, " _disibledevent="); //过滤其它控件的on...事件 
            //html = regex6.Replace(html, ""); //过滤img 
            //html = html.Replace("</strong>", "");
            //html = html.Replace("<strong>", "");
            return html;
        }
    }

    public static class IntExtension
    {
        [DebuggerStepThrough]
        public static string ToFileSize(this int fileSize)
        {
            string m_strSize = "";
            long FactSize = 0;
            FactSize = fileSize;
            if (FactSize < 1024.00)
                m_strSize = FactSize.ToString("F2") + " Byte";
            else if (FactSize >= 1024.00 && FactSize < 1048576)
                m_strSize = (FactSize / 1024.00).ToString("F2") + " K";
            else if (FactSize >= 1048576 && FactSize < 1073741824)
                m_strSize = (FactSize / 1024.00 / 1024.00).ToString("F2") + " M";
            else if (FactSize >= 1073741824)
                m_strSize = (FactSize / 1024.00 / 1024.00 / 1024.00).ToString("F2") + " G";
            return m_strSize;
        }

        [DebuggerStepThrough]
        public static string ToAudioLength(this int audioLength)
        {
            string m_strSize = "";
            double size = audioLength;
            if (size < 60)
                m_strSize = size + "秒";
            else if (size >= 60 && size < 3600)
                m_strSize = (size / 60).ToString("F2") + " 分钟";
            else if (size >= 3600)
                m_strSize = (size / 3600).ToString("F2") + "小时";
            return m_strSize;
        }
        [DebuggerStepThrough]
        public static string ToFileSize(this long fileSize)
        {
            string m_strSize = "";
            long FactSize = 0;
            FactSize = fileSize;
            if (FactSize < 1024.00)
                m_strSize = FactSize.ToString("F2") + " Byte";
            else if (FactSize >= 1024.00 && FactSize < 1048576)
                m_strSize = (FactSize / 1024.00).ToString("F2") + " K";
            else if (FactSize >= 1048576 && FactSize < 1073741824)
                m_strSize = (FactSize / 1024.00 / 1024.00).ToString("F2") + " M";
            else if (FactSize >= 1073741824)
                m_strSize = (FactSize / 1024.00 / 1024.00 / 1024.00).ToString("F2") + " G";
            return m_strSize;
        }

        [DebuggerStepThrough]
        public static string ToAudioLength(this long audioLength)
        {
            string m_strSize = "";
            double size = audioLength;
            if (size < 60)
                m_strSize = size + "秒";
            else if (size >= 60 && size < 3600)
                m_strSize = (size / 60).ToString("F2") + " 分钟";
            else if (size >= 3600)
                m_strSize = (size / 3600).ToString("F2") + "小时";
            return m_strSize;
        }
    }
}
